import asyncio
import gc
import json
from nertivia import cache_nertivia_data
import aiohttp
import nest_asyncio
import requests

import nertivia.message

nest_asyncio.apply()

# Set API endpoints
MAIN_URL = "https://nertivia.net/"
URL = "https://nertivia.net/api/channels/"
URL_MSG = "https://nertivia.net/api/messages/"
URL_STA = "https://nertivia.net/api/settings/status"

headers = {}

loop = asyncio.get_event_loop()

session = aiohttp.ClientSession()


def get_sid(token):
    """
    Obtain the sid from a given token, returns None if failed connection or other error preventing success
    Do not use manually
    """
    r = requests.get(url=str(URL + "app"), headers={'Accept': 'text/plain',
                                                    'authorization': token,
                                                    'Content-Type': 'application/json;charset=utf-8'})
    cookie = r.headers.get('set-cookie')

    # If successful, then the cookie was set
    if cookie:
        return cookie.split("connect.sid=", 1)[1].strip("; Path=/; HttpOnly")
    return None


def generate_headers(token):
    """
    Generates a header using a provided token and sets it as a global variable
    Do not use manually
    """
    global headers
    headers = {'Accept': 'text/plain',
               'authorization': token,
               'Content-Type': 'application/json;charset=utf-8',
               'Cookie': f'connect.sid={get_sid(token)}'}


async def fetch_server(server_id):
    """
    Asynchronous function which perform an API call to retrieve a server from its ID
    """

    session = aiohttp.ClientSession()
    res = await session.get(url=str(f'{MAIN_URL}/api/servers/{server_id}'),
                            headers=headers)
    await session.close()

    # Reminder : 2XX is a success
    # If unsuccessful we return the error message
    if res.status != 200:
        return res.content

    # However, if succesful return the json data that was returned and transform it into its python equivalent
    return await res.json()


async def fetch_channel(channel_id):
    """
    Asynchronous function that will perform an API call to retrieve a channel from its ID
    """
    session = aiohttp.ClientSession()
    res = await session.get(url=str(f'{URL}{channel_id}'), headers=headers)
    await session.close()

    # Reminder : 2XX is a success
    # If unsuccessful we return the error message
    if res.status != 200:
        return res.content

    # However, if succesful return the json data that was returned and transform it into its python equivalent
    return await res.json()


async def fetch_user(user_id):
    """
    Asynchronous function which performs an API call to retrieve a user from their ID
    """
    session = aiohttp.ClientSession()
    res = await session.get(url=str(f'{MAIN_URL}/api/user/{user_id}'),
                            headers=headers)
    await session.close()

    # Reminder : 2XX is a success
    # If unsuccessful we return the error message
    if res.status != 200:
        return res.content

    # However, if successful return the json data that was returned and transform it into its python equivalent
    return await res.json()


class HTTPClient:
    """
    Object representing the HTTPClient, do not instantiate manually
    Handles the bridge between Nertivia.py and the Nertivia servers
    """
    def __init__(self, **kwargs):
        """
        Prepare the HTTPClient instance
        """
        self.token = None
        self.user = {}
        self._servers = {}
        self._users = {}

        # If a token was passed as a named argument then set it as a self value for ease of acces
        if kwargs.get("token"):
            self.token = kwargs.get("token")

        # If a socket ip was given, use it to ready all of the endpoints
        if kwargs.get("socket_ip"):
            global MAIN_URL, URL, URL_MSG, URL_STA
            socket_ip = kwargs.get("socket_ip")
            MAIN_URL = f"{socket_ip}"
            URL = f"{socket_ip}/api/channels/"
            URL_MSG = f"{socket_ip}/api/messages/"
            URL_STA = f"{socket_ip}/api/settings/status"

    def set_token(self, token):
        """
        Set the token to the one provided, regardless of whether or not one was already set
        After which, generate new headers to fit with the new token
        """
        self.token = token
        generate_headers(token)

    def clear(self):
        """
        Clean up the memory, reset the dictionary entries for users, servers and the bot itself
        """
        self.user = {}
        self._servers = {}
        self._users = {}

        gc.collect()  # make sure it's memory efficient

    @property
    def servers(self):
        """
        Synchronously returns a list of all cached servers
        """
        return list(self._servers.values())

    def _get_server(self, server_id):
        """
        Returns a cached server using its ID
        If the server isn't cached, then `None` is returned
        """
        return self._servers.get(server_id)

    def _add_server(self, server):
        """
        Add a server to the cache, using its ID as the dictionary key and the server object being the value
        """
        self._servers[server.id] = server

    def _remove_server(self, server):
        """
        Remove from the cache a server using its ID
        For some reason server object is expected and not id (Fix if no dependencies)
        """

        # Remove the server from cache, then delete the argument despite no apparent further reference (Useless line ?)
        self._servers.pop(server.id, None)
        del server
        gc.collect()

    async def delete_message(self, message_id, channel_id):
        """
        Asynchronously delete a message using its ID, and the channel ID in which it is located
        """
        res = await session.delete(url=str(URL_MSG + str(message_id) + '/channels/' + str(channel_id)),
                                   headers=headers)

        # Reminder : if 2XX then it is a success
        if res.status != 200:
            return res.content

    async def edit_message(self, message_id, channel_id, content: str):
        """
        Edit a message using its ID, and the channel ID in which it is located
        Third argument is the content to replace it with
        """
        res = await session.patch(url=str(URL_MSG + str(message_id) + '/channels/' + str(channel_id)),
                                  headers=headers,
                                  data=json.dumps({'message': content}))
        if res.status != 200:
            return res.content

    async def send_message(self, channel_id, content: str):
        """
        Send a message in a channel which is found by its ID
        Second argument is a string containing the message to send
        """
        res = await session.post(url=str(URL_MSG + '/channels/' + str(channel_id)),
                                 data=json.dumps({"message": content}),
                                 headers=headers)
        # Reminder : if 2XX then it is a success
        if res.status != 200:
            return res
        # If successful return...Success message ? Not sure
        return res

    async def get_message(self, message_id, channel_id):
        """
        Asynchronously retrieve a message using its ID and the ID of the channel in which it is located
        """
        res = await session.get(url=str(f'{MAIN_URL}/api/messages/{message_id}/channels/{channel_id}'),
                                headers=headers)

        # Reminder : if 2XX then it is a success
        if res.status != 200:
            # If the response is unsuccessful and has a content attribute then we return it
            # Needs doc reading but a content attribute should always be present to specify success/error message
            try:
                if "content" in res:
                    return res.content
            # Error shouldn't be possible unless res is None
            except Exception:
                pass
            return None

        # Return a message object for easy manipulation
        return nertivia.message.Message({'message': await res.json()})

    def get_channel(self, channel_id):
        """
        Synchronously fetch a channel using its ID with an API call
        Channels are not cached ?
        Returns a nertivia.Channel object
        """
        try:
            res = asyncio.run(fetch_channel(channel_id))
        except asyncio.TimeoutError:
            res = asyncio.run(fetch_channel(channel_id))
        return nertivia.Channel(res)

    def get_user(self, user_id, force_cache: bool = False):
        """
        Synchronously obtain a user from cache, if the user is not cached then perform an API call to retrieve them
        `force_cache` may be set to True to have the function return None rather than perform an API call
        """

        if str(user_id) in cache_nertivia_data.users:
            return cache_nertivia_data.users[str(user_id)]
        elif not force_cache:
            return nertivia.User(asyncio.run(fetch_user(user_id)))
        return None

    def get_server(self, server_id, force_cache: bool = False):
        """
        Synchronously obtain a server from cache, if the server is not cached then perform an API call to retrieve it
        `force_cache` may be set to True to have the function return None rather than perform an API call
        """

        if str(server_id) in cache_nertivia_data.guilds:
            return cache_nertivia_data.guilds[str(server_id)]
        elif not force_cache:
            return nertivia.Server(asyncio.run(fetch_server(server_id)))

        return None



import sys

from .client import Client
if sys.version_info >= (3, 5):  # pragma: no cover
    from .asyncio_client import AsyncClient
else:  # pragma: no cover
    AsyncClient = None

__version__ = '4.2.0'

__all__ = ['__version__', 'Client']



"""JSON-compatible module with sane defaults."""

from json import *  # noqa: F401, F403
from json import loads as original_loads


def _safe_int(s):
    if len(s) > 100:
        raise ValueError('Integer is too large')
    return int(s)


def loads(*args, **kwargs):
    if 'parse_int' not in kwargs:  # pragma: no cover
        kwargs['parse_int'] = _safe_int
    return original_loads(*args, **kwargs)



import nertivia
import nertivia.bot
from nertivia import http

# asyncio and nertivia.http unused ?

# Set the different endpoints that'll be used
URL = "https://nertivia.net/api/messages/channels/"
URL_MSG = "https://nertivia.net/api/messages/"
URL_STA = "https://nertivia.net/api/settings/status"


class Message:
    """
    Object class used to store a Messages information
    """
    def __init__(self, message, **kwargs):
        """
        Set different variables that'll be used later on for ease of access
        """
        # If a message object is passed then use its information to create this Message object
        # Otherwise create a new one from nothing
        if kwargs.get('http') and isinstance(kwargs['http'], http.HTTPClient):
            self.http = kwargs['http']
        else:
            self.http = http.HTTPClient()
        if "message" in message:
            self.id: int = message['message']['messageID']
            if "message" in message['message']:
                self.content: str = message['message']['message']
            else:
                self.content: str = ""
            if "files" in message['message']:
                self.files = message["message"]["files"]
            self.channel: nertivia.Channel = self.http.get_channel(message["message"]["channelID"])
            self.author: str = message['message']['creator']['username'] + '@' + message['message']['creator']['tag']

        else:
            self.id: int = message['messageID']
            self.content: str = message['message']
            self.channel: nertivia.Channel = self.http.get_channel(message["channelID"])
            self.author: str = message['creator']['username'] + '@' + message['creator']['tag']

        self.server: nertivia.Server = self.channel.server

    def __repr__(self):
        """
        Return a representation of the class Instance
        """
        return f"<id={self.id} content='{self.content}' channel={self.channel.__repr__()} " \
               f"server={self.server.__repr__()} author='{self.author}'>"

    @property
    def _id(self):
        """
        Return the ID of the message
        """
        return self.id

    @property
    def _author(self) -> nertivia.user:
        """
        Return the author of the message
        """
        return self.author

    async def edit(self, content):
        """
        Asynchronous function to edit a message, takes a channel object and a message content
        """
        await self.http.edit_message(self.id, self.channel.id, content)

    async def send(self, message):
        """
        Send a new message using a message
        Do not call manually
        """
        message = await self.http.send_message(self.channel.id, message)
        message_json = await message.json()
        return Message({"message": message_json["messageCreated"]})

    async def delete(self):
        """
        Asynchronously delete the message
        """
        await self.http.delete_message(self.id, self.channel.id)



import nertivia
from nertivia import http
from nertivia import message

# Official Nertivia Server endpoints, Consider having the first part be variable and changeable from a central location
URL = "https://nertivia.net/api/messages/channels/"
URL_MSG = "https://nertivia.net/api/messages/"
URL_STA = "https://nertivia.net/api/settings/status"


class Channel(object):
    """
    Class defining a Channel, not expected to be manually instantiated
    """
    def __init__(self, channel, **kwargs):
        """
        Creates a new channel object, takes in a dictionary containing full Channel information
        """
        
        # Allow for user-specific HTTPClient object.
        if kwargs.get('http') and isinstance(kwargs['http'], http.HTTPClient):
            self.http = kwargs['http']
        else:
            # Set the channel http attribute to the http object to simplify sending or retrieving of messages
            self.http = http.HTTPClient()

        # Set all of the properties of the channel in a self.x blob, consider cleaning up if possible
        # If a Channel object is provided in the arguments, then just extract all of its information
        if "channel" in channel:
            self.id = channel["channel"]["channelID"]
            self.name = channel["channel"]["name"]
            self.status = channel["channel"]["status"]
            self.name = channel["channel"]["name"]
            self.server: nertivia.Server = self.http.get_server(channel["channel"]["server_id"])
            self.last_messaged = channel["channel"]["timestamp"]
            self._channel = channel["channel"]

        # If it is a new Channel object, then we have less information to set and use what was provided
        else:
            self.id = channel["channelID"]
            self.name = channel["name"]
            self.server = self.http.get_server(channel["server_id"])
            if "timestamp" in channel:
                self.last_messaged = channel["timestamp"]
            self._channel = channel

    def __repr__(self):
        """
        Allow for channel to be used in strings
        """
        # Self.server removed .__repr__ as repr will be returned when used in this context
        return f"<id={self.id} name='{self.name}' server=<{self.server}>>"

    async def send(self, message_content):
        """
        Asynchronously send a message in a channel
        """
        # Throw it to the http object

        return_message = await self.http.send_message(self.id, message_content)
        message_json = await return_message.json()
        return message.Message({"message": message_json["messageCreated"]})

    async def get_message(self, message_id):
        """
        Asynchronously retrieve a message from a channel using its id
        """
        # Throw it to the http object
        return await self.http.get_message(message_id, self.id)



import urllib

from . import packet


class Payload(object):
    """Engine.IO payload."""
    max_decode_packets = 16

    def __init__(self, packets=None, encoded_payload=None):
        self.packets = packets or []
        if encoded_payload is not None:
            self.decode(encoded_payload)

    def encode(self, jsonp_index=None):
        """Encode the payload for transmission."""
        encoded_payload = ''
        for pkt in self.packets:
            if encoded_payload:
                encoded_payload += '\x1e'
            encoded_payload += pkt.encode(b64=True)
        if jsonp_index is not None:
            encoded_payload = '___eio[' + \
                              str(jsonp_index) + \
                              ']("' + \
                              encoded_payload.replace('"', '\\"') + \
                              '");'
        return encoded_payload

    def decode(self, encoded_payload):
        """Decode a transmitted payload."""
        self.packets = []

        if len(encoded_payload) == 0:
            return

        # JSONP POST payload starts with 'd='
        if encoded_payload.startswith('d='):
            encoded_payload = urllib.parse.parse_qs(
                encoded_payload)['d'][0]

        encoded_packets = encoded_payload.split('\x1e')
        if len(encoded_packets) > self.max_decode_packets:
            raise ValueError('Too many packets in payload')
        self.packets = [packet.Packet(encoded_packet=encoded_packet)
                        for encoded_packet in encoded_packets]



from collections import OrderedDict
from typing import Dict

# The following dictionaries hold all cached information relative to their name
from .user import User
from .server import Server


class LimitedCache(OrderedDict):
    """
    Holds limited amount of information, acts as a cache object for messages objects
    Size of it can be edited during initialisation
    """

    def __init__(self, maxsize=500, /, *args, **kwds):
        """
        MaxSize affects the maximum size of the cache (main use -> messages to reduce RAM usage)
        """
        self.maxsize = maxsize
        super().__init__(*args, **kwds)

    def __getitem__(self, key):
        """
        Synchronously obtain the requested item depending on the key
        """
        return super().__getitem__(key)

    def __setitem__(self, key, value):
        """
        Synchronously set an item of the cache to a given value
        """
        super().__setitem__(key, value)
        # Bit of logic to max sure that the maxsize is respected
        if len(self) > self.maxsize:
            oldest = next(iter(self))
            del self[oldest]


users: OrderedDict[str, User] = LimitedCache(maxsize=300)
members: OrderedDict[str, User] = LimitedCache(maxsize=800)
guilds: Dict[str, Server] = {"0": Server(None)}
user: User = User(None)

messages = LimitedCache(maxsize=1000)



import base64
from engineioN import json as _json

(OPEN, CLOSE, PING, PONG, MESSAGE, UPGRADE, NOOP) = (0, 1, 2, 3, 4, 5, 6)
packet_names = ['OPEN', 'CLOSE', 'PING', 'PONG', 'MESSAGE', 'UPGRADE', 'NOOP']

binary_types = (bytes, bytearray)


class Packet(object):
    """Engine.IO packet."""

    json = _json

    def __init__(self, packet_type=NOOP, data=None, encoded_packet=None):
        self.packet_type = packet_type
        self.data = data
        if isinstance(data, str):
            self.binary = False
        elif isinstance(data, binary_types):
            self.binary = True
        else:
            self.binary = False
        if self.binary and self.packet_type != MESSAGE:
            raise ValueError('Binary packets can only be of type MESSAGE')
        if encoded_packet:
            self.decode(encoded_packet)

    def encode(self, b64=False):
        """Encode the packet for transmission."""
        if self.binary:
            if b64:
                encoded_packet = 'b' + base64.b64encode(self.data).decode(
                    'utf-8')
            else:
                encoded_packet = self.data
        else:
            encoded_packet = str(self.packet_type)
            if isinstance(self.data, str):
                encoded_packet += self.data
            elif isinstance(self.data, dict) or isinstance(self.data, list):
                encoded_packet += self.json.dumps(self.data,
                                                  separators=(',', ':'))
            elif self.data is not None:
                encoded_packet += str(self.data)
        return encoded_packet

    def decode(self, encoded_packet):
        """Decode a transmitted package."""
        self.binary = isinstance(encoded_packet, binary_types)
        b64 = not self.binary and encoded_packet[0] == 'b'
        if b64:
            self.binary = True
            self.packet_type = MESSAGE
            self.data = base64.b64decode(encoded_packet[1:])
        else:
            if self.binary and not isinstance(encoded_packet, bytes):
                encoded_packet = bytes(encoded_packet)
            if self.binary:
                self.packet_type = MESSAGE
                self.data = encoded_packet
            else:
                self.packet_type = int(encoded_packet[0])
                try:
                    self.data = self.json.loads(encoded_packet[1:])
                    if isinstance(self.data, int):
                        # do not allow integer payloads, see
                        # github.com/miguelgrinberg/python-engineioN/issues/75
                        # for background on this decision
                        raise ValueError
                except ValueError:
                    self.data = encoded_packet[1:]



import requests
from .user import User
import socketioN

# Nertivia server endpoint
SOCKET_IP = "https://nertivia.net/"
sio = socketioN.Client(engineio_logger=True, logger=True)


class Client:
    """
    Client object, ancestor of Bot object found in bot.py
    DEPRECATED -> USE bot.py
    """
    def __init__(self):
        """
        Initiate the Client, set variables for future uses
        """
        self._listeners = {}
        self.token = None
        self.sio = sio
        self.headers = {'Accept': 'text/plain',
                        'authorization': self.token,
                        'Content-Type': 'application/json;charset=utf-8'}

    def _get_sio(self):
        """
        Return sio (socketioN.Client)
        """
        return self.sio

    def login(self, token):
        """
        Log into Nertivia using the Token to access the bot account
        """
        print("Deprecated client! Please use nertivia.bot instead!")

        # Obtain the token from the provided string
        self.token = token.strip()
        # Connect to the Nertivia severs
        self.sio.connect(SOCKET_IP, namespaces=['/'], transports=['websocket'])

        # Event listener, when asked return the authentication header
        @self.sio.event
        def connect():
            sio.emit('authentication', {'token': token})

        return self.sio

    def get_user(self, userID):
        """
        Obtain a user from their id
        This is a direct API call and doesn't cache | Beware & use bot.py
        """
        print("Deprecated client! Please use nertivia.bot instead!")
        headers = {'Accept': 'text/plain',
                   'authorization': self.token,
                   'Content-Type': 'application/json;charset=utf-8'}
        r1 = requests.get(url=f'https://nertivia.net/api/user/{userID}', headers=headers)
        user = r1.json()

        user = User(user)

        return user

    def event(self, *args):
        """
        Handles event decorators
        Only basic events are available as this is deprecated -> use bot.py
        """
        print("Deprecated client! Please use nertivia.bot instead!")
        if args[0].__name__ == "on_ready":
            return self.sio.on("success")(args[0])
        if args[0].__name__ == "on_message":
            return self.sio.on("success")(args[0])
        if args[0].__name__ == "on_quit":
            return self.sio.on("disconnect")(args[0])



import asyncio
import logging
import traceback

import socketioN
from .http import HTTPClient
from .user import User

default_logger = logging.getLogger('nertivia')

SOCKET_IP = "https://nertivia.net"

token = None
user = None


class Bot:
    """
    Bot class object, represents the interface which represents the bot user
    """
    global token, default_logger

    def __init__(self, **args) -> None:
        """
        :param args:
        Expects named arguments, wrapped in **args
        Possible arguments are :
            `test` for a local Nertivia Test Server
            `debug` to activate debug mode (increased information logging)
            None for regular operation
        """

        # Socket IP is a string representing the url to the Nertivia Server (By default the official Server,nertivia.net
        # user is a User class object from user.py, set to None but expected to be defined before the Bot class being
        # - Initiated
        global SOCKET_IP, user

        # Check if a local server is being used to test Nertivia features (Address hardcoded to be the default one)
        if args.get("test"):
            # noinspection HttpUrlsUsage
            SOCKET_IP = "http://server.localtest.me"

        # Check if the library user desires mass information logging for debug purposes, if so initialises the client
        # Accordingly
        if args.get("debug"):
            self.sio = socketioN.AsyncClient(logger="True", engineio_logger="True")
        # Otherwise the bot is started normally
        else:
            self.sio = socketioN.AsyncClient()

        # Set self.http to the HTTP client connection object for future use
        self.http = HTTPClient(socket_ip=SOCKET_IP)

        # Note : Consider moving around, if i forgot to remove this check feasibility to move this upwards
        import nertivia.cache_nertivia_data
        self.user: User = nertivia.cache_nertivia_data.user
        self.headers = {'Accept': 'text/plain',
                        'authorization': token,
                        'Content-Type': 'application/json;charset=utf-8'}

    def _get_sio(self):
        """
        :return:
        Synchronous function returning the AsyncClient object from socketioN
        """
        return self.sio

    async def main(self, new_token: str) -> None:
        """
        :param new_token:
        :return None:

        Asynchronous function initiating a (new) connection to Nertivia using the Token (str) passed as an argument
        """

        # Start connection to SOCKET_IP, then send authentication header
        await self.sio.connect(SOCKET_IP, transports=['websocket'])
        await self.sio.emit('authentication', {'token': new_token})
        self.sio.on('update_bot_user', self.update_bot_user)
        a_sio = self.sio

        # Wait for a response, if an authentication error has occurred auth_err() is called
        @a_sio.event
        def auth_err(data):
            print("Invalid Token")

        @a_sio.event
        async def connect():
            print("reconnected to server!")
            await self.sio.emit('authentication', {'token': new_token})
        await a_sio.wait()

    def update_bot_user(self, data):
        """
        :param data:
        :return:

        Update the Bot User: replace it by a new/updated `bot` Instance
        """
        global user
        self.user = data

    def login(self, new_token: str):
        """
        :param new_token:
        :return:

        Have the script login into an account using a provided Token
        """

        # Set global token to the provided token, before then setting the token globally to the http connection object
        # After which, call `main` method using asyncio.run as it is an asynchronous function
        global token
        token = new_token
        self.http.set_token(token)
        asyncio.run(self.main(new_token))

        return self.sio

    async def get_user(self, userID):
        """
        :param userID:
        :return:

        Obtain a user object from user.py from cache preferably, but if not cached then fetch it
        """
        return await self.http.get_user(userID)

    def on(self, event, handler=None, namespace=None):
        """
        :param event:
        :param handler:
        :param namespace:
        :return:

        Event Handler, creates Handlers if non existent
        """

        # Exceptions may happen
        try:
            namespace = namespace or '/'

            # Set a Handler to an event, handlers are set in the dictionary mess inside of sio.handlers
            # noinspection PyShadowingNames
            def set_handler(handler):
                # If namespace doesn't exist in the handlers dictionary, we create a dictionary for it before moving on
                if namespace not in self.sio.handlers:
                    self.sio.handlers[namespace] = {}

                # If the event doesn't exist inside of the handler, inside of the namespace then we create a list for it
                if event not in self.sio.handlers[namespace]:
                    self.sio.handlers[namespace][event] = []

                # Finally we append the handler to the correct list
                self.sio.handlers[namespace][event].append(handler)

                return handler

            # If the event handler isn't set then we return the function that links a handler to an event once
            # The handler is created (so that it may be passed as an argument)
            if handler is None:
                return set_handler

            # Otherwise, we just call set_handler
            set_handler(handler)

        # In case something unexpected happened, we catch it and print the error
        except Exception:
            # noinspection PyArgumentList
            traceback.print_exc()

    def event(self, *args):
        """
        :param args:
        :return:

        Handler for all event.x decorators
        Also this is a nice if wall
        """
        HandledEvents = {"on_ready": "on_ready", "on_message": "receiveMessage", "on_quit": "disconnect",
                         "on_status_change": "member:custom_status_change", "on_message_delete": "delete_message",
                         "on_message_edit": "update_message"}

        for Key, Value in HandledEvents.items():
            if args[0].__name__ == Key:
                return self.on(Value)(args[0])



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


requirements: list = ["setuptools", "requests", "nest_asyncio", "six>=1.9.0", "websocket-client>=0.54.0",
                      "websockets>=7.0", "aiohttp>=3.4", "bidict"]

setuptools.setup(
    name='nertivia',  # How you named your package folder (MyLib)
    version='0.2.8',  # Start with a small number and increase it with every change you make
    license='apache-2.0',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Python API Wrapper for Nertivia to make bots',  # Give a short description about your library
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Nertivia-PY/Nertivia.py/",
    author='FluxedScript',  # Type in your name
    keywords=['API Wrapper', 'SIMPLE', 'PYTHON', "nertivia"],  # Keywords that define your package best
    package_dir={"socketioN": "socketioN"},
    packages=setuptools.find_namespace_packages(include=['nertivia', "socketioN", "engineioN"]),
    install_requires=requirements,
    extras_require={
        'client': [
            'requests>=2.21.0',
            'websocket-client>=0.54.0',
        ],
        'asyncio_client': [
            'aiohttp>=3.4',
            'websockets>=7.0',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
    ],
    python_requires=">=3.6",
)



class User(object):
    """
    Object representing a user
    """
    def __init__(self, user, **kwargs):
        """
        Instantiate variables to be used later on
        """
        # If the user is not cached
        if user is not None:
            if "user" not in user:
                user = {"user": user}

            # Check whether the user ID is stored under `uniquerID` or `id`
            if "uniqueID" in user["user"]:
                self.id = user['user']['uniqueID']
            else:
                self.id = user['user']['id']

            self.username = user['user']['username']
            self.tag = user['user']['tag']

            # Set it to false first in case it doesn't exist, same story as using an &&
            self.bot = False if "bot" not in user["user"] else user["user"]["bot"]

            self.avatar_url = "https://nertivia.net/api/avatars/{}".format(user['user']['avatar'])

            # Set the user representation as Username#Tag
            self.user = "{}@{}".format(user['user']['username'], user['user']['tag'])

    def __repr__(self):
        """
        Returns a representation of the object
        """
        return f"<id={self.id} username='{self.username}' tag='{self.tag}' " \
               f"avatar_url='{self.avatar_url}' user='{self.user}' bot={self.bot}>"

    @property
    def _id(self):
        """
        Returns the id of the User
        """
        return self.id

    @property
    def _name(self):
        """
        Returns the name of the user
        """
        return self.username

    @property
    def _avatar_url(self):
        """
        Returns the url of the avatar of the user
        """
        return self.avatar_url

    @property
    def _user(self):
        """
        Returns the representation of the users name
        """
        return self.user



import nertivia
from nertivia import http


class Server(object):
    """
    Object representing a server
    """
    def __init__(self, server, **kwargs):
        """
        Set basic variables for ease of use
        """
        self.http = http.HTTPClient()
        if server is not None:
            self.id = server['server_id']
            self.name = server['name']
            self.default_channel: nertivia.Channel = server['default_channel_id']

    def __repr__(self):
        """
        Returns a representation of the class
        """
        return f"<id={self.id} name='{self.name}' default_channel=<{self.default_channel}>>"

    @property
    def _id(self):
        """
        Returns the server id
        """
        return self.id

    @property
    def _name(self):
        """
        Returns the server name
        """
        return self.name
